﻿using Nevron.Nov.Diagram;
using Nevron.Nov.Diagram.Shapes;
using Nevron.Nov.Graphics;
using Nevron.Nov.UI;
using Nevron.Nov.WpfControls;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using Nevron.Nov.UI;

namespace NevronTestLib
{
    public static class TestGenerator
    {
        private static Dictionary<int, NShape> Nodes = new Dictionary<int, NShape>();
        private static NBasicShapesFactory BasicShapeFactory = new NBasicShapesFactory();
        private static NConnectorShapesFactory ConnectorFactory = new NConnectorShapesFactory();

        private const int TABLE_CELL_MARGIN_RIGHT = 70;
        private const int TABLE_CELL_MARGIN_BOTTOM = 70;
        private const string DEFAULT_SYMBOL_IMAGE = @"..\..\Resources\desig1.jpg";
        private const int DESCRIPTION_NODE_OFFSET_X = 110;
        private const int DEFAULT_WIDTH = 70;
        private const int DEFAULT_HEIGHT = 70;
        private const int GROUP_SPACING = 20;
        
        public static NDrawingViewControl generate()
        {
            NDrawingViewControl control = new NDrawingViewControl();
            
            CreateNodes(control);
            CreateConnectors(control);
            return control;
        }

        private static void CreateNodes(NDrawingViewControl control)
        {
            using (FileStream stream = File.Open(DEFAULT_SYMBOL_IMAGE, FileMode.Open))
            {
                NImage image = NImage.FromStream(stream);
                for (int i = 0; i < 10; i++)
                {
                    for (int j = 0; j < 10; j++)
                    {
                        NShape imageNode = CreateImageNode(image, "" + i);
                        NShape descriptionNode = CreateDescriptionNode("" + i);
                        Nodes.Add(i*10+j, imageNode);

                        NPoint p = MapTableSize2Pixel(i, j);
                        NGroup group = CreateGroup(p, imageNode, descriptionNode);

                        control.Widget.Document.Content.ActivePage.Items.AddChild(group);
                    }
                }
            }
        }

        private static NGroup CreateGroup(NPoint p, NShape imageNode, NShape descriptionNode)
        {
            NGroup group = new NGroup();
            group.Shapes.Add(imageNode);
            group.Shapes.Add(descriptionNode);
            group.Tag = imageNode.Tag;
            group.SelectionMode = ENGroupSelectionMode.GroupFirst;
            group.SetBounds(new NRectangle(p.X - DEFAULT_WIDTH + GROUP_SPACING, p.Y, 2 * DEFAULT_WIDTH, DEFAULT_HEIGHT));

            return group;
        }


        private static NShape CreateImageNode(NImage image, string id)
        {
            NShape shape = BasicShapeFactory.CreateShape(ENBasicShapes.Rectangle);
            shape.SetBounds(new NRectangle(DEFAULT_WIDTH, 0, DEFAULT_WIDTH, DEFAULT_HEIGHT));
            shape.Geometry.Fill = new NImageFill((NImage)image.DeepClone());
            //shape.AllowInplaceEdit = false;
            shape.Tag = id;
            return shape;
        }

        private static NShape CreateDescriptionNode(string text)
        {
            NShape shape = BasicShapeFactory.CreateShape(ENBasicShapes.Text);
            //shape.Text = text;
            shape.SetBounds(new NRectangle(0, 0, DEFAULT_WIDTH, DEFAULT_HEIGHT));
            shape.Geometry.ShowFill = false;
            // shape.AllowInplaceEdit = false;
            return shape;
        }

        private static NPoint MapTableSize2Pixel(double x, double y)
        {
            return new NPoint(x * (DESCRIPTION_NODE_OFFSET_X + TABLE_CELL_MARGIN_RIGHT),
                            y * (DESCRIPTION_NODE_OFFSET_X + TABLE_CELL_MARGIN_BOTTOM));
        }

        private static void CreateConnectors(NDrawingViewControl control)
        {
            for(int i = 0; i < 50; i++)
            {
                NShape connectorShape = CreateConnector();
                control.Widget.Document.Content.ActivePage.Items.AddChild(connectorShape);
            }
        }


        private static NShape CreateConnector()
        {
            Random rand = new Random();
            NShape source = Nodes[rand.Next(80)];
            NShape destination = Nodes[rand.Next(80)];

            NShape connectorShape = null;
            connectorShape = ConnectorFactory.CreateShape(ENConnectorShapes.RoutableConnector);

            connectorShape.Geometry.CornerRounding = 20;
            if (source == destination)
            {
                NPort fromPort = source.Ports.GetPortByName("Right");
                NPort toPort = destination.Ports.GetPortByName("Top");
                connectorShape.GlueBeginToPort(fromPort);
                connectorShape.GlueEndToPort(toPort);

                connectorShape.Geometry.CornerRounding = 5;
            }
            else
            {
                connectorShape.GlueBeginToShape(source);
                connectorShape.GlueEndToShape(destination);
            }
            //connectorShape.AllowSelect = false;
            //connectorShape.AllowInplaceEdit = false;

            return connectorShape;
        }

    }
}
